@@ -14,6 +14,7 @@
 #include <avr/interrupt.h>
 #include <string.h>
 #include <avr/io.h>
+#include <avr/wdt.h>
 #include <avr/pgmspace.h>
 #include <stdlib.h>
 #include <avr/eeprom.h>
@@ -417,6 +418,7 @@ void programcmd(unsigned char seqnum)
                         //page mode, all modern chips
                         for(i=0;i<nbytes;i++)
                         {
+                                wd_kick();
                                 // The Low/High byte selection bit is
                                 // bit number 3. Set high byte for uneven bytes
                                 if(addressing_is_word && i&1) {
@@ -512,9 +514,9 @@ void programcmd(unsigned char seqnum)
                         nbytes=280;
                 }
                 //
-                wd_kick();
 		for(i=0;i<nbytes;i++)
 		{
+                        wd_kick();
 			//Select Low or High-Byte
 			if(addressing_is_word && i&1) {
 				spi_mastertransmit(tmp|(1<<3));
@@ -559,6 +561,7 @@ void programcmd(unsigned char seqnum)
                         if (msg_buf[1] == (ci + 1)){
                                 msg_buf[2] = tmp;
                         }
+			delay_ms(5);
                 }
                 answerlen = 4;
                 // msg_buf[0] = CMD_READ_FUSE_ISP; or CMD_READ_LOCK_ISP or ...
@@ -586,6 +589,7 @@ void programcmd(unsigned char seqnum)
                         }else{
                                 spi_mastertransmit(msg_buf[cj+4]);
                         }
+			delay_ms(5);
                 }
                 // padd with zero:
                 while(ci<tmp){
@@ -620,7 +624,7 @@ signed char terminalmode_readnum(unsigned char chr_nl){
 		// we expect \r\n as line end
 		lineendcnt=2;
 	}
-	ch=uart_getchar();
+	ch=uart_getchar(1);
 	while(ch != '\r' && ch !='\n' && digits){
 		if ((ch >= '0') && (ch <= '9')){
 			if (rval==-1){
@@ -630,7 +634,7 @@ signed char terminalmode_readnum(unsigned char chr_nl){
 			rval=rval*10 + (ch  - (unsigned char)'0');
 			digits--;
 		}
-		ch=uart_getchar();
+		ch=uart_getchar(1);
 	}
 	// user just pressed return or enter just 1 digit
 	if (ch == '\r' || ch == '\n'){
@@ -638,7 +642,7 @@ signed char terminalmode_readnum(unsigned char chr_nl){
 	}
 	// get \n or \r\n after a number was entered:
 	while(lineendcnt){
-		ch=uart_getchar();
+		ch=uart_getchar(1);
 		lineendcnt--;
 	}
 	return(rval);
@@ -656,7 +660,7 @@ void terminalmode(unsigned char chr_nl)
 	// Init terminal
 	uart_sendstr_p(terminal_init);
 	//
-	uart_sendstr_P("AvrUsb500-1.3 terminal mode\r\n");
+	uart_sendstr_P("AvrUsb500-1.4 terminal mode\r\n");
 	uart_sendstr_P("Enter SW Version Major [");
 	utoa(CONFIG_PARAM_SW_MAJOR,(char *)msg_buf, 10);
 	uart_sendstr((char *)msg_buf);
@@ -716,7 +720,11 @@ int main(void)
                 CONFIG_PARAM_SW_MAJOR=eeprom_read_byte(EEPROM_MAJOR);
         }
         while(1){
-                ch=uart_getchar();
+                if (msgparsestate==MSG_IDLE){
+                        ch=uart_getchar(1);
+                }else{
+                        ch=uart_getchar(0);
+                }
                 // parse message according to appl. note AVR068 table 3-1:
                 if (msgparsestate==MSG_IDLE && ch == MESSAGE_START){
                         msgparsestate=MSG_WAIT_SEQNUM;
@@ -783,6 +791,7 @@ int main(void)
                         cksum^=ch;
                         msg_buf[i]=ch;
                         i++;
+			wdt_reset(); //wd_kick();
                         if (i==msglen){
                                 msgparsestate=MSG_WAIT_CKSUM;
                         }
@@ -791,7 +800,7 @@ int main(void)
                 if (msgparsestate==MSG_WAIT_CKSUM){
                         if (ch==cksum && msglen > 0){
                                 // message correct, process it
-                                wd_kick();
+                                wdt_reset(); //wd_kick();
                                 programcmd(seqnum);
                         }else{
                                 msg_buf[0] = ANSWER_CKSUM_ERROR;
